# Copyright (c) 2019, Suprock Technologies
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
# OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

# Asphodel protocol version 2.0.5
# NOTE: use the functions in asphodel_version.h to access the protocol version
ASPHODEL_PROTOCOL_VERSION_MAJOR = 0x02
ASPHODEL_PROTOCOL_VERSION_MINOR = 0x01
ASPHODEL_PROTOCOL_VERSION_SUBMINOR = 0x00

# USB class/subclass defines
# use one ASPHODEL_PROTOCOL_TYPE_* as the USB protocol definition
ASPHODEL_USB_CLASS = 0xFF  # 0xFF: vendor specific USB class
ASPHODEL_USB_SUBCLASS = 0x01  # 0x01: Generic Sensor

# protocol types to define various implementations
ASPHODEL_PROTOCOL_TYPE_BASIC = 0x00  # basic (minimum) implementation
ASPHODEL_PROTOCOL_TYPE_RF_POWER = 0x01  # RF Power protocol extension
ASPHODEL_PROTOCOL_TYPE_RADIO = 0x02  # radio interface
ASPHODEL_PROTOCOL_TYPE_REMOTE = 0x04  # remote interface
ASPHODEL_PROTOCOL_TYPE_BOOTLOADER = 0x08  # firmware bootloader

# general information commands
CMD_GET_PROTOCOL_VERSION = 0x00
CMD_GET_BOARD_INFO = 0x01
CMD_GET_USER_TAG_LOCATIONS = 0x02
CMD_GET_BUILD_INFO = 0x03
CMD_GET_BUILD_DATE = 0x04
CMD_GET_CHIP_FAMILY = 0x05
CMD_GET_CHIP_MODEL = 0x06
CMD_GET_CHIP_ID = 0x07

# NVM commands
CMD_GET_NVM_SIZE = 0x08
CMD_ERASE_NVM = 0x09
CMD_WRITE_NVM = 0x0A
CMD_READ_NVM = 0x0B

# Flush/Reinit communication pipes
CMD_FLUSH = 0x0C

# reset commands
CMD_RESET = 0x0D
CMD_GET_BOOTLOADER_INFO = 0x0E
CMD_BOOTLOADER_JUMP = 0x0F

# LED commands
CMD_GET_RGB_COUNT = 0x10
CMD_GET_RGB_VALUES = 0x11
CMD_SET_RGB = 0x12
CMD_SET_RGB_INSTANT = 0x13
CMD_GET_LED_COUNT = 0x14
CMD_GET_LED_VALUE = 0x15
CMD_SET_LED = 0x16
CMD_SET_LED_INSTANT = 0x17

# stream commands
CMD_GET_STREAM_COUNT_AND_ID = 0x20
CMD_GET_STREAM_CHANNELS = 0x21
CMD_GET_STREAM_FORMAT = 0x22
CMD_ENABLE_STREAM = 0x23
CMD_WARM_UP_STREAM = 0x24
CMD_GET_STREAM_STATUS = 0x25
CMD_GET_STREAM_RATE_INFO = 0x26

# channel commands
CMD_GET_CHANNEL_COUNT = 0x30
CMD_GET_CHANNEL_NAME = 0x31
CMD_GET_CHANNEL_INFO = 0x32
CMD_GET_CHANNEL_COEFFICIENTS = 0x33
CMD_GET_CHANNEL_CHUNK = 0x34
CMD_CHANNEL_SPECIFIC = 0x35

# power supply check commands
CMD_GET_SUPPLY_COUNT = 0x40
CMD_GET_SUPPLY_NAME = 0x41
CMD_GET_SUPPLY_INFO = 0x42
CMD_CHECK_SUPPLY = 0x43

# control variable commands
CMD_GET_CTRL_VAR_COUNT = 0x50
CMD_GET_CTRL_VAR_NAME = 0x51
CMD_GET_CTRL_VAR_INFO = 0x52
CMD_GET_CTRL_VAR = 0x53
CMD_SET_CTRL_VAR = 0x54

# settings commands
CMD_GET_SETTING_COUNT = 0x60
CMD_GET_SETTING_NAME = 0x61
CMD_GET_SETTING_INFO = 0x62
CMD_GET_SETTING_DEFAULT = 0x63
CMD_GET_CUSTOM_ENUM_COUNTS = 0x64
CMD_GET_CUSTOM_ENUM_VALUE_NAME = 0x65
CMD_GET_SETTING_CATEGORY_COUNT = 0x66
CMD_GET_SETTING_CATEGORY_NAME = 0x67
CMD_GET_SETTING_CATERORY_SETTINGS = 0x68

# device mode commands
CMD_SET_DEVICE_MODE = 0x70
CMD_GET_DEVICE_MODE = 0x71

# RF Power commands (only supported by ASPHODEL_PROTOCOL_TYPE_RF_POWER)
CMD_ENABLE_RF_POWER = 0x80
CMD_GET_RF_POWER_STATUS = 0x81
CMD_GET_RF_POWER_CTRL_VARS = 0x82
CMD_RESET_RF_POWER_TIMEOUT = 0x83

# Radio commands (only supported by ASPHODEL_PROTOCOL_TYPE_RADIO)
CMD_STOP_RADIO = 0x90
CMD_START_RADIO_SCAN = 0x91
CMD_GET_RADIO_SCAN_RESULTS = 0x92
CMD_CONNECT_RADIO = 0x93
CMD_GET_RADIO_STATUS = 0x94
CMD_GET_RADIO_CTRL_VARS = 0x95
CMD_GET_RADIO_DEFAULT_SERIAL = 0x96
CMD_START_RADIO_SCAN_BOOT = 0x97
CMD_CONNECT_RADIO_BOOT = 0x98
CMD_GET_RADIO_EXTRA_SCAN_RESULTS = 0x99
CMD_GET_RADIO_SCAN_POWER = 0x9F

# Remote commands (only supported by ASPHODEL_PROTOCOL_TYPE_REMOTE)
CMD_STOP_REMOTE = 0x9A
CMD_RESTART_REMOTE = 0x9B
CMD_GET_REMOTE_STATUS = 0x9C
CMD_RESTART_REMOTE_APP = 0x9D
CMD_RESTART_REMOTE_BOOT = 0x9E
# NOTE: 0x9F is grouped above with the radio commands

# Bootloader commands (only supported by ASPHODEL_PROTOCOL_TYPE_BOOTLOADER)
CMD_BOOTLOADER_START_PROGRAM = 0xA0
CMD_GET_BOOTLOADER_PAGE_INFO = 0xA1
CMD_GET_BOOTLOADER_BLOCK_SIZES = 0xA2
CMD_START_BOOTLOADER_PAGE = 0xA3
CMD_WRITE_BOOTLOADER_CODE_BLOCK = 0xA4
CMD_FINISH_BOOTLOADER_PAGE = 0xA5
CMD_VERIFY_BOOTLOADER_PAGE = 0xA6

# Commands for low-level hardware interaction. Used for testing.
CMD_GET_GPIO_PORT_COUNT = 0xE0
CMD_GET_GPIO_PORT_NAME = 0xE1
CMD_GET_GPIO_PORT_INFO = 0xE2
CMD_GET_GPIO_PORT_VALUES = 0xE3
CMD_SET_GPIO_PORT_MODES = 0xE4
CMD_DISABLE_GPIO_PORT_OVERRIDES = 0xE5
CMD_GET_BUS_COUNTS = 0xE6
CMD_SET_SPI_CS_MODE = 0xE7
CMD_DO_SPI_TRANSFER = 0xE8
CMD_DO_I2C_WRITE = 0xE9
CMD_DO_I2C_READ = 0xEA
CMD_DO_I2C_WRITE_READ = 0xEB
CMD_DO_RADIO_FIXED_TEST = 0xEC
CMD_DO_RADIO_SWEEP_TEST = 0xED

# Commands for querying device info regions. Used for testing.
CMD_GET_INFO_REGION_COUNT = 0xF0
CMD_GET_INFO_REGION_NAME = 0xF1
CMD_GET_INFO_REGION = 0xF2

# Misc internal testing commands. Seriously, don't use these.
CMD_GET_STACK_INFO = 0xF3

# Commands for echoing various bytes back to the host. Used for testing.
CMD_ECHO_RAW = 0xFC
CMD_ECHO_TRANSACTION = 0xFD
CMD_ECHO_PARAMS = 0xFE

# Error reply
CMD_REPLY_ERROR = 0xFF

# Error codes
ERROR_CODE_UNSPECIFIED = 0x01
ERROR_CODE_MALFORMED_COMMAND = 0x02
ERROR_CODE_UNIMPLEMENTED_COMMAND = 0x03
ERROR_CODE_BAD_CMD_LENGTH = 0x04
ERROR_CODE_BAD_ADDRESS = 0x05
ERROR_CODE_BAD_INDEX = 0x06
ERROR_CODE_INVALID_DATA = 0x07
ERROR_CODE_UNSUPPORTED = 0x08
ERROR_CODE_BAD_STATE = 0x09
ERROR_CODE_I2C_ERROR = 0x0A
ERROR_CODE_INCOMPLETE = 0x0B

# Unit types
UNIT_TYPE_NONE = 0  # should not be converted to any other unit
UNIT_TYPE_LSB = 1  # LSB (directly from an ADC or similar)
UNIT_TYPE_PERCENT = 2  # percent (unitless * 100)
UNIT_TYPE_VOLT = 3  # voltage
UNIT_TYPE_AMPERE = 4  # current
UNIT_TYPE_WATT = 5  # power
UNIT_TYPE_OHM = 6  # electrical resistance
UNIT_TYPE_CELSIUS = 7  # temperature
UNIT_TYPE_PASCAL = 8  # pressure
UNIT_TYPE_NEWTON = 9  # force
UNIT_TYPE_M_PER_S = 10  # velocity
UNIT_TYPE_M_PER_S2 = 11  # acceleration / gravity
UNIT_TYPE_DB = 12  # logarithmic (unitless)
UNIT_TYPE_DBM = 13  # logarithmic (power)
UNIT_TYPE_STRAIN = 14  # strain (unitless)
UNIT_TYPE_HZ = 15  # frequency
UNIT_TYPE_SECOND = 16  # time
UNIT_TYPE_LSB_PER_CELSIUS = 17  # LSB per unit temperature
UNIT_TYPE_KG_PER_S = 18  # mass flow
UNIT_TYPE_L_PER_S = 19  # volumetric flow
UNIT_TYPE_NEWTON_METER = 20  # torque
UNIT_TYPE_METER = 21  # length
UNIT_TYPE_GRAM = 22  # mass
UNIT_TYPE_COUNT = 23

# Channel Types
CHANNEL_TYPE_LINEAR = 0
CHANNEL_TYPE_NTC = 1
CHANNEL_TYPE_ARRAY = 2
CHANNEL_TYPE_SLOW_STRAIN = 3
CHANNEL_TYPE_FAST_STRAIN = 4
CHANNEL_TYPE_SLOW_ACCEL = 5
CHANNEL_TYPE_PACKED_ACCEL = 6
CHANNEL_TYPE_COMPOSITE_STRAIN = 7
CHANNEL_TYPE_LINEAR_ACCEL = 8
CHANNEL_TYPE_COUNT = 9

# Supply check result bit masks
ASPHODEL_SUPPLY_LOW_BATTERY = 0x01
ASPHODEL_SUPPLY_TOO_LOW = 0x02
ASPHODEL_SUPPLY_TOO_HIGH = 0x04

# Setting Types
SETTING_TYPE_BYTE = 0
SETTING_TYPE_BOOLEAN = 1
SETTING_TYPE_UNIT_TYPE = 2
SETTING_TYPE_CHANNEL_TYPE = 3
SETTING_TYPE_BYTE_ARRAY = 4
SETTING_TYPE_STRING = 5
SETTING_TYPE_INT32 = 6
SETTING_TYPE_INT32_SCALED = 7
SETTING_TYPE_FLOAT = 8
SETTING_TYPE_FLOAT_ARRAY = 9
SETTING_TYPE_CUSTOM_ENUM = 10
SETTING_TYPE_COUNT = 11

# GPIO pin modes
GPIO_PIN_MODE_HI_Z = 0
GPIO_PIN_MODE_PULL_DOWN = 1
GPIO_PIN_MODE_PULL_UP = 2
GPIO_PIN_MODE_LOW = 3
GPIO_PIN_MODE_HIGH = 4

# SPI CS modes
SPI_CS_MODE_LOW = 0
SPI_CS_MODE_HIGH = 1
SPI_CS_MODE_AUTO_TRANSFER = 2
SPI_CS_MODE_AUTO_BYTE = 3

# Strain channel specific commands
STRAIN_SET_OUTPUTS = 0x01

# Accel channel specific commands
ACCEL_ENABLE_SELF_TEST = 0x01
